@@ -8,9 +8,8 @@
 #include "config.h"
 #endif /* HAVE_CONFIG_H */
 
-#if defined (USE_PAM) && defined (UAM_DHX2)
-
 #include <errno.h>
+#include <gcrypt.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -24,10 +23,6 @@
 #include <pam/pam_appl.h>
 #endif
 
-#ifdef HAVE_LIBGCRYPT
-#include <gcrypt.h>
-#endif /* HAVE_LIBGCRYPT */
-
 #include <atalk/afp.h>
 #include <atalk/globals.h>
 #include <atalk/logger.h>
@@ -77,8 +72,10 @@ static int dh_params_generate (unsigned int bits) {
 
     /* Version check should be the very first call because it
        makes sure that important subsystems are initialized. */
-    if (!gcry_check_version (GCRYPT_VERSION)) {
-        LOG(log_error, logtype_uams, "PAM DHX2: libgcrypt versions mismatch. Need: %s", GCRYPT_VERSION);
+    if (!gcry_check_version (UAM_NEED_LIBGCRYPT_VERSION)) {
+        LOG(log_error, logtype_uams,
+            "PAM DHX2: libgcrypt versions mismatch. Needs: %s Has: %s",
+            UAM_NEED_LIBGCRYPT_VERSION, gcry_check_version (NULL));
         result = AFPERR_MISC;
         goto error;
     }
@@ -135,7 +132,7 @@ error:
  * echo off means password.
  */
 static int PAM_conv (int num_msg,
-#if !defined(__svr4__)
+#ifdef HAVE_PAM_CONV_CONST_PAM_MESSAGE
                      const struct pam_message **msg,
 #else
                      struct pam_message **msg,
@@ -362,7 +359,7 @@ static int pam_login(void *obj, struct passwd **uam_pwd,
     if ((unsigned long) ibuf & 1) /* pad to even boundary */
         ++ibuf;
 
-    return (login(obj, username, ulen, uam_pwd, ibuf, ibuflen, rbuf, rbuflen));
+    return login(obj, username, ulen, uam_pwd, ibuf, ibuflen, rbuf, rbuflen);
 }
 
 /* ----------------------------- */
@@ -397,7 +394,7 @@ static int pam_login_ext(void *obj, char *uname, struct passwd **uam_pwd,
     memcpy(username, uname +2, len );
     username[ len ] = '\0';
 
-    return (login(obj, username, ulen, uam_pwd, ibuf, ibuflen, rbuf, rbuflen));
+    return login(obj, username, ulen, uam_pwd, ibuf, ibuflen, rbuf, rbuflen);
 }
 
 /* -------------------------------- */
@@ -421,7 +418,7 @@ static int logincont1(void *obj _U_, char *ibuf, size_t ibuflen, char *rbuf, siz
 
     /* Packet size should be: Session ID + Ma + Encrypted client nonce */
     if (ibuflen != 2 + PRIMEBITS/8 + 16) {
-        LOG(log_error, logtype_uams, "DHX2: Paket length not correct");
+        LOG(log_error, logtype_uams, "DHX2: Packet length not correct");
         ret = AFPERR_PARAM;
         goto error_noctx;
     }
@@ -452,6 +449,7 @@ static int logincont1(void *obj _U_, char *ibuf, size_t ibuflen, char *rbuf, siz
     K_MD5hash = calloc(1, K_hash_len = gcry_md_get_algo_dlen(GCRY_MD_MD5));
     if (K_MD5hash == NULL) {
         ret = AFPERR_MISC;
+        free(K_bin);
         goto error_noctx;
     }
     gcry_md_hash_buffer(GCRY_MD_MD5, K_MD5hash, K_bin, PRIMEBITS/8);
@@ -561,8 +559,15 @@ static int loginasroot(const char *adminauthuser, const char **hostname, int sta
     /* solaris craps out if PAM_TTY and PAM_RHOST aren't set. */
     pam_set_item(pamh, PAM_TTY, "afpd");
     pam_set_item(pamh, PAM_RHOST, *hostname);
-    if ((PAM_error = pam_authenticate(pamh, 0)) != PAM_SUCCESS)
+    if ((PAM_error = pam_authenticate(pamh, 0)) != PAM_SUCCESS) {
+        LOG(log_info, logtype_uams, "DHX2 loginasroot: error authenticating with PAM");
         goto exit;
+    }
+
+    if ((PAM_error = pam_acct_mgmt(pamh, 0)) != PAM_SUCCESS) {
+        LOG(log_info, logtype_uams, "DHX2 loginasroot: error validating PAM account");
+        goto exit;
+    }
 
     LOG(log_warning, logtype_uams, "DHX2: Authenticated as \"%s\"", adminauthuser);
 
@@ -578,7 +583,7 @@ static int logincont2(void *obj_in, struct passwd **uam_pwd,
     int ret = AFPERR_MISC;
     int PAM_error;
     const char *hostname = NULL;
-    gcry_mpi_t retServerNonce;
+    gcry_mpi_t retServerNonce = NULL;
     gcry_cipher_hd_t ctx;
     gcry_error_t ctxerror;
     char *utfpass = NULL;
@@ -587,7 +592,7 @@ static int logincont2(void *obj_in, struct passwd **uam_pwd,
 
     /* Packet size should be: Session ID + ServerNonce + Passwd buffer (evantually +10 extra bytes, see Apples Docs) */
     if ((ibuflen != 2 + 16 + 256) && (ibuflen != 2 + 16 + 256 + 10)) {
-        LOG(log_error, logtype_uams, "DHX2: Paket length not correct: %u. Should be 274 or 284.", ibuflen);
+        LOG(log_error, logtype_uams, "DHX2: Packet length not correct: %u. Should be 274 or 284.", ibuflen);
         ret = AFPERR_PARAM;
         goto error_noctx;
     }
@@ -644,10 +649,6 @@ static int logincont2(void *obj_in, struct passwd **uam_pwd,
     }
     PAM_password = utfpass;
 
-#ifdef DEBUG
-    LOG(log_maxdebug, logtype_default, "DHX2: password: %s", PAM_password);
-#endif
-
     /* Set these things up for the conv function */
 
     ret = AFPERR_NOTAUTH;
@@ -763,13 +764,13 @@ static int changepw_1(void *obj, char *uname,
 
     /* Remember it now, use it in changepw_3 */
     PAM_username = uname;
-    return( dhx2_setup(obj, ibuf, ibuflen, rbuf, rbuflen) );
+    return dhx2_setup(obj, ibuf, ibuflen, rbuf, rbuflen);
 }
 
 static int changepw_2(void *obj,
                       char *ibuf, size_t ibuflen, char *rbuf, size_t *rbuflen)
 {
-    return( logincont1(obj, ibuf, ibuflen, rbuf, rbuflen) );
+    return logincont1(obj, ibuf, ibuflen, rbuf, rbuflen);
 }
 
 static int changepw_3(void *obj _U_,
@@ -781,7 +782,7 @@ static int changepw_3(void *obj _U_,
     uid_t uid;
     pam_handle_t *lpamh;
     const char *hostname = NULL;
-    gcry_mpi_t retServerNonce;
+    gcry_mpi_t retServerNonce = NULL;
     gcry_cipher_hd_t ctx;
     gcry_error_t ctxerror;
 
@@ -791,7 +792,7 @@ static int changepw_3(void *obj _U_,
 
     /* Packet size should be: Session ID + ServerNonce + 2*Passwd buffer */
     if (ibuflen != 2 + 16 + 2*256) {
-        LOG(log_error, logtype_uams, "DHX2: Paket length not correct");
+        LOG(log_error, logtype_uams, "DHX2: Packet length not correct");
         ret = AFPERR_PARAM;
         goto error_noctx;
     }
@@ -863,18 +864,34 @@ static int changepw_3(void *obj _U_,
     uam_afpserver_option(obj, UAM_OPTION_CLIENTNAME, (void *) &hostname, NULL);
     pam_set_item(lpamh, PAM_RHOST, hostname);
     uid = geteuid();
-    seteuid(0);
+    if (seteuid(0) < 0) {
+      LOG(log_error, logtype_uams, "DHX2 Chgpwd: could not seteuid(%i)", 0);
+    }
     PAM_error = pam_authenticate(lpamh,0);
     if (PAM_error != PAM_SUCCESS) {
         LOG(log_info, logtype_uams, "DHX2 Chgpwd: error authenticating with PAM");
-        seteuid(uid);
+        if (seteuid(uid) < 0) {
+            LOG(log_error, logtype_uams, "DHX2 Chgpwd: could not seteuid(%i)", uid);
+        }
+        pam_end(lpamh, PAM_error);
+        ret = AFPERR_NOTAUTH;
+        goto error_ctx;
+    }
+    PAM_error = pam_acct_mgmt(lpamh, 0);
+    if (PAM_error != PAM_SUCCESS) {
+        LOG(log_info, logtype_uams, "DHX2 Chgpwd: error validating PAM account");
+        if (seteuid(uid) < 0) {
+            LOG(log_error, logtype_uams, "DHX2 Chgpwd: could not seteuid(%i)", uid);
+        }
         pam_end(lpamh, PAM_error);
         ret = AFPERR_NOTAUTH;
         goto error_ctx;
     }
     PAM_password = ibuf;
     PAM_error = pam_chauthtok(lpamh, 0);
-    seteuid(uid); /* un-root ourselves. */
+    if (seteuid(uid) < 0) {
+      LOG(log_error, logtype_uams, "DHX2 Chgpwd: could not seteuid(%i)", uid);
+    }
     memset(ibuf, 0, 512);
     if (PAM_error != PAM_SUCCESS) {
         LOG(log_info, logtype_uams, "DHX2 Chgpwd: error changing pw with PAM");
@@ -926,7 +943,7 @@ static int dhx2_changepw(void *obj _U_, char *uname,
     return ret;
 }
 
-static int uam_setup(void *obj, const char *path)
+static int uam_setup(void *obj _U_, const char *path)
 {
     if (uam_register(UAM_SERVER_LOGIN_EXT, path, "DHX2", pam_login,
                      pam_logincont, pam_logout, pam_login_ext) < 0)
@@ -967,5 +984,3 @@ UAM_MODULE_EXPORT struct uam_export uams_dhx2_pam = {
     UAM_MODULE_VERSION,
     uam_setup, uam_cleanup
 };
-
-#endif /* USE_PAM && UAM_DHX2 */
