@@ -30,11 +30,6 @@
 
 #define PASSWDLEN 8
 
-#ifndef MIN
-#define MIN(a,b) ((a) < (b) ? (a) : (b))
-#endif /* MIN */
-
-
 /* Static variables used to communicate between the conversation function
  * and the server_login function
  */
@@ -52,7 +47,7 @@ extern UAM_MODULE_EXPORT void append(struct papfile *, const char *, int);
  * echo off means password.
  */
 static int PAM_conv (int num_msg,
-#if !defined(__svr4__)
+#ifdef HAVE_PAM_CONV_CONST_PAM_MESSAGE
                      const struct pam_message **msg,
 #else
                      struct pam_message **msg,
@@ -223,7 +218,7 @@ static int pam_login(void *obj, struct passwd **uam_pwd,
 
     len = (unsigned char) *ibuf++;
     if ( len > ulen ) {
-	return( AFPERR_PARAM );
+	return AFPERR_PARAM;
     }
 
     memcpy(username, ibuf, len );
@@ -233,7 +228,7 @@ static int pam_login(void *obj, struct passwd **uam_pwd,
 
     if ((unsigned long) ibuf & 1)  /* pad character */
       ++ibuf;
-    return (login(obj, username, ulen, uam_pwd, ibuf, ibuflen, rbuf, rbuflen));
+    return login(obj, username, ulen, uam_pwd, ibuf, ibuflen, rbuf, rbuflen);
 }
 
 /* ----------------------------- */
@@ -257,12 +252,12 @@ static int pam_login_ext(void *obj, char *uname, struct passwd **uam_pwd,
     len = ntohs(temp16);
 
     if (!len || len > ulen ) {
-        return( AFPERR_PARAM );
+        return AFPERR_PARAM;
     }
     memcpy(username, uname +2, len );
     username[ len ] = '\0';
 
-    return (login(obj, username, ulen, uam_pwd, ibuf, ibuflen, rbuf, rbuflen));
+    return login(obj, username, ulen, uam_pwd, ibuf, ibuflen, rbuf, rbuflen);
 }
 
 /* logout */
@@ -293,7 +288,8 @@ static int pam_changepw(void *obj _U_, char *username,
 
     /* Set these things up for the conv function */
     PAM_username = username;
-    PAM_password = pw;
+    memcpy(PAM_password, pw, PASSWDLEN);
+    PAM_password[PASSWDLEN] = '\0';
 
     PAM_error = pam_start("netatalk", username, &PAM_conversation,
 			  &lpamh);
@@ -304,10 +300,22 @@ static int pam_changepw(void *obj _U_, char *username,
 
     /* we might need to do this as root */
     uid = geteuid();
-    seteuid(0);
+    if (seteuid(0) < 0) {
+      LOG(log_error, logtype_uams, "pam_changepw: could not seteuid(%i)", 0);
+    }
     PAM_error = pam_authenticate(lpamh,0);
     if (PAM_error != PAM_SUCCESS) {
-      seteuid(uid);
+      if (seteuid(uid) < 0) {
+        LOG(log_error, logtype_uams, "pam_changepw: could not seteuid(%i)", uid);
+      }
+      pam_end(lpamh, PAM_error);
+      return AFPERR_NOTAUTH;
+    }
+    PAM_error = pam_acct_mgmt(lpamh, 0);
+    if (PAM_error != PAM_SUCCESS) {
+      if (seteuid(uid) < 0) {
+        LOG(log_error, logtype_uams, "pam_changepw: could not seteuid(%i)", uid);
+      }
       pam_end(lpamh, PAM_error);
       return AFPERR_NOTAUTH;
     }
@@ -319,7 +327,9 @@ static int pam_changepw(void *obj _U_, char *username,
 
     /* this really does need to be done as root */
     PAM_error = pam_chauthtok(lpamh, 0);
-    seteuid(uid); /* un-root ourselves. */
+    if (seteuid(uid) < 0) {
+      LOG(log_error, logtype_uams, "pam_changepw: could not seteuid(%i)", uid);
+    }
     memset(pw, 0, PASSWDLEN);
     memset(ibuf, 0, PASSWDLEN);
     if (PAM_error != PAM_SUCCESS) {
@@ -336,15 +346,16 @@ static int pam_changepw(void *obj _U_, char *username,
 static int pam_printer(char *start, char *stop, char *username, struct papfile *out)
 {
     int PAM_error;
-    char	*data, *p, *q;
-    char	password[PASSWDLEN + 1] = "\0";
+    char *data;
+    const char *p;
+    const char *q;
     static const char *loginok = "0\r";
     struct passwd *pwd;
 
     data = (char *)malloc(stop - start + 1);
     if (!data) {
 	LOG(log_info, logtype_uams,"Bad Login ClearTxtUAM: malloc");
-	return(-1);
+	return -1;
     }
 
     strlcpy(data, start, stop - start + 1);
@@ -359,13 +370,13 @@ static int pam_printer(char *start, char *stop, char *username, struct papfile *
     if ((p = strchr(data, '(' )) == NULL) {
 	LOG(log_info, logtype_uams,"Bad Login ClearTxtUAM: username not found in string");
 	free(data);
-	return(-1);
+	return -1;
     }
     p++;
     if ((q = strstr(p, ") (" )) == NULL) {
 	LOG(log_info, logtype_uams,"Bad Login ClearTxtUAM: username not found in string");
 	free(data);
-	return(-1);
+	return -1;
     }
     memcpy(username, p, MIN(UAM_USERNAMELEN, q - p) );
 
@@ -374,9 +385,18 @@ static int pam_printer(char *start, char *stop, char *username, struct papfile *
     if ((q = strrchr(p, ')' )) == NULL) {
 	LOG(log_info, logtype_uams,"Bad Login ClearTxtUAM: password not found in string");
 	free(data);
-	return(-1);
+	return -1;
     }
-    memcpy(password, p, MIN(PASSWDLEN, (q - p)) );
+
+    /* Allocate memory for the global PAM_password */
+    PAM_password = (char *)malloc(PASSWDLEN + 1);
+    if (!PAM_password) {
+        LOG(log_info, logtype_uams,"Bad Login ClearTxtUAM: malloc failed for password");
+        free(data);
+        return -1;
+    }
+    memset(PAM_password, 0, PASSWDLEN + 1);
+    memcpy(PAM_password, p, MIN(PASSWDLEN, (q - p)));
 
     /* Done copying username and password, clean up */
     free(data);
@@ -384,16 +404,19 @@ static int pam_printer(char *start, char *stop, char *username, struct papfile *
     if (( pwd = uam_getname(NULL, username, strlen(username))) == NULL ) {
         LOG(log_info, logtype_uams, "Bad Login ClearTxtUAM: ( %s ) not found ",
             username);
-        return(-1);
+        free(PAM_password);
+        PAM_password = NULL;
+        return -1;
     }
 
     if (uam_checkuser(pwd) < 0) {
         /* syslog of error happens in uam_checkuser */
-        return(-1);
+        free(PAM_password);
+        PAM_password = NULL;
+        return -1;
     }
 
     PAM_username = username;
-    PAM_password = password;
 
     PAM_error = pam_start("netatalk", username, &PAM_conversation,
                           &pamh);
@@ -402,7 +425,9 @@ static int pam_printer(char *start, char *stop, char *username, struct papfile *
 			username, pam_strerror(pamh, PAM_error));
         pam_end(pamh, PAM_error);
         pamh = NULL;
-        return(-1);
+        free(PAM_password);
+        PAM_password = NULL;
+        return -1;
     }
 
     pam_set_item(pamh, PAM_TTY, "papd");
@@ -413,7 +438,9 @@ static int pam_printer(char *start, char *stop, char *username, struct papfile *
 			username, pam_strerror(pamh, PAM_error));
         pam_end(pamh, PAM_error);
         pamh = NULL;
-        return(-1);
+        free(PAM_password);
+        PAM_password = NULL;
+        return -1;
     }
 
     PAM_error = pam_acct_mgmt(pamh, 0);
@@ -422,7 +449,9 @@ static int pam_printer(char *start, char *stop, char *username, struct papfile *
 			username, pam_strerror(pamh, PAM_error));
         pam_end(pamh, PAM_error);
         pamh = NULL;
-        return(-1);
+        free(PAM_password);
+        PAM_password = NULL;
+        return -1;
     }
 
     PAM_error = pam_open_session(pamh, 0);
@@ -431,7 +460,9 @@ static int pam_printer(char *start, char *stop, char *username, struct papfile *
 			username, pam_strerror(pamh, PAM_error));
         pam_end(pamh, PAM_error);
         pamh = NULL;
-        return(-1);
+        free(PAM_password);
+        PAM_password = NULL;
+        return -1;
     }
 
     /* Login successful, but no need to hang onto it,
@@ -442,11 +473,15 @@ static int pam_printer(char *start, char *stop, char *username, struct papfile *
     pam_end(pamh, 0);
     pamh = NULL;
 
-    return(0);
+    /* Before returning, free the allocated password */
+    free(PAM_password);
+    PAM_password = NULL;
+
+    return 0;
 }
 
 
-static int uam_setup(void *obj, const char *path)
+static int uam_setup(void *obj _U_, const char *path)
 {
   if (uam_register(UAM_SERVER_LOGIN_EXT, path, "Cleartxt Passwrd",
 		   pam_login, NULL, pam_logout, pam_login_ext) < 0)
