@@ -7,6 +7,7 @@
 #include "config.h"
 #endif /* HAVE_CONFIG_H */
 
+#include <netdb.h>
 #include <string.h>
 #include <errno.h>
 #include <signal.h>
@@ -22,9 +23,6 @@
 #include <atalk/ddp.h>
 #include <atalk/netddp.h>
 
-#ifdef HAVE_NETDB_H
-#include <netdb.h>
-#endif /* HAVE_NETDB_H */
 #include  "nbp_conf.h"
 
 /* FIXME/SOCKLEN_T: socklen_t is a unix98 feature. */
@@ -47,7 +45,7 @@ int nbp_rgstr( struct sockaddr_at *sat, const char *obj, const char *type, const
 
     if ( nbp_lookup( obj, type, zone, &nn, 1, &sat->sat_addr ) > 0 ) {
         errno = EADDRINUSE;
-	return( -1 );
+	return -1;
     }
 
     memset(&to, 0, sizeof(to));
@@ -70,7 +68,7 @@ int nbp_rgstr( struct sockaddr_at *sat, const char *obj, const char *type, const
     data += SZ_NBPTUPLE;
 
     if ( obj ) {
-	if (( cc = strlen( obj )) > NBPSTRLEN ) return( -1 );
+	if (( cc = strlen( obj )) > NBPSTRLEN ) return -1;
 	*data++ = cc;
 	memcpy( data, obj, cc );
 	data += cc;
@@ -79,7 +77,7 @@ int nbp_rgstr( struct sockaddr_at *sat, const char *obj, const char *type, const
     }
 
     if ( type ) {
-	if (( cc = strlen( type )) > NBPSTRLEN ) return( -1 );
+	if (( cc = strlen( type )) > NBPSTRLEN ) return -1;
 	*data++ = cc;
 	memcpy( data, type, cc );
 	data += cc;
@@ -88,7 +86,7 @@ int nbp_rgstr( struct sockaddr_at *sat, const char *obj, const char *type, const
     }
 
     if ( zone ) {
-	if (( cc = strlen( zone )) > NBPSTRLEN ) return( -1 );
+	if (( cc = strlen( zone )) > NBPSTRLEN ) return -1;
 	*data++ = cc;
 	memcpy( data, zone, cc );
 	data += cc;
@@ -97,7 +95,7 @@ int nbp_rgstr( struct sockaddr_at *sat, const char *obj, const char *type, const
 	*data++ = '*'; /* default zone */
     }
 
-    
+
     if ( nbp_port == 0 ) {
 	if (( se = getservbyname( "nbp", "ddp" )) == NULL ) {
 	    nbp_port = 2;
@@ -107,7 +105,7 @@ int nbp_rgstr( struct sockaddr_at *sat, const char *obj, const char *type, const
     }
     to.sat_port = nbp_port;
 
-    if ( netddp_sendto( s, nbp_send, data - nbp_send, 0, 
+    if ( netddp_sendto( s, nbp_send, data - nbp_send, 0,
 			(struct sockaddr *)&to,
 			sizeof( struct sockaddr_at )) < 0 ) {
         goto register_err;
@@ -135,13 +133,13 @@ int nbp_rgstr( struct sockaddr_at *sat, const char *obj, const char *type, const
 
     data = nbp_recv;
     if ( *data++ != DDPTYPE_NBP ) {
-	return( -1 );
+	return -1;
     }
     memcpy( &nh, data, SZ_NBPHDR );
     if ( nh.nh_op != NBPOP_OK ) {
         return -1;
     }
-    return( 0 );
+    return 0;
 
 register_err:
     netddp_close(s);
