@@ -73,7 +73,7 @@ int sys_get_easize(VFS_FUNC_ARGS_EA_GETSIZE)
 	LOG(log_debug, logtype_afpd, "sys_get_easize(%s): file is already opened", uname);
 	ret = sys_fgetxattr(fd, attruname, rbuf +4, 0);
     } else {
-	if ((oflag & O_NOFOLLOW) ) {
+	if (oflag & O_NOFOLLOW ) {
     	    ret = sys_lgetxattr(uname, attruname, rbuf +4, 0);
 	}
 	else {
@@ -170,6 +170,10 @@ int sys_get_eacontent(VFS_FUNC_ARGS_EA_GETCONTENT)
         return AFPERR_ACCESS;
 #endif
 
+    /* Protect special attributes set by Netatalk server */
+    if (!strncmp(attruname, AD_EA_META, AD_EA_META_LEN))
+        return AFPERR_ACCESS;
+
     /* Start building reply packet */
 
     if (maxreply <= MAX_REPLY_EXTRA_BYTES) {
@@ -197,7 +201,7 @@ int sys_get_eacontent(VFS_FUNC_ARGS_EA_GETCONTENT)
 	LOG(log_debug, logtype_afpd, "sys_get_eacontent(%s): file is already opened", uname);
 	ret = sys_fgetxattr(fd, attruname, rbuf +4, maxreply + extra);
     } else {
-	if ((oflag & O_NOFOLLOW) ) {
+	if (oflag & O_NOFOLLOW ) {
     	    ret = sys_lgetxattr(uname, attruname, rbuf +4, maxreply + extra);
 	}
 	else {
@@ -271,8 +275,8 @@ int sys_get_eacontent(VFS_FUNC_ARGS_EA_GETCONTENT)
  *
  * Copies names of all EAs of uname as consecutive C strings into rbuf.
  * Increments *rbuflen accordingly.
- * We hide the adouble:ea extended attributes here, but we currently
- * allow reading, writing and deleteting them.
+ * We hide the adouble:ea extended attributes here, we do not
+ * allow reading, writing and deleting them.
  */
 int sys_list_eas(VFS_FUNC_ARGS_EA_LIST)
 {
@@ -280,7 +284,7 @@ int sys_list_eas(VFS_FUNC_ARGS_EA_LIST)
     int     ret, len, nlen;
     char    *buf;
     char    *ptr;
-    struct adouble ad, *adp;
+    struct adouble ad _U_, *adp _U_;
 
     buf = malloc(ATTRNAMEBUFSIZ);
     if (!buf)
@@ -291,7 +295,7 @@ int sys_list_eas(VFS_FUNC_ARGS_EA_LIST)
 	LOG(log_debug, logtype_afpd, "sys_list_eas(%s): file is already opened", uname);
 	ret = sys_flistxattr(fd, uname, buf, ATTRNAMEBUFSIZ);
     } else {
-	if ((oflag & O_NOFOLLOW)) {
+	if (oflag & O_NOFOLLOW) {
     	    ret = sys_llistxattr(uname, buf, ATTRNAMEBUFSIZ);
 	}
 	else {
@@ -381,6 +385,10 @@ int sys_set_ea(VFS_FUNC_ARGS_EA_SET)
         return AFPERR_ACCESS;
 #endif
 
+    /* Protect special attributes set by Netatalk server. Silently fail in order for file copies to work */
+    if (!strncmp(attruname, AD_EA_META, AD_EA_META_LEN))
+        return AFP_OK;
+
     /*
      * Buffer for a copy of the xattr plus one byte in case
      * AFPVOL_EA_SAMBA is used
@@ -393,9 +401,9 @@ int sys_set_ea(VFS_FUNC_ARGS_EA_SET)
     eabuf[attrsize] = 0;
 
     attr_flag = 0;
-    if ((oflag & O_CREAT) )
+    if (oflag & O_CREAT)
         attr_flag |= XATTR_CREATE;
-    else if ((oflag & O_TRUNC) )
+    else if (oflag & O_TRUNC)
         attr_flag |= XATTR_REPLACE;
 
     if (vol->v_flags & AFPVOL_EA_SAMBA) {
@@ -407,7 +415,7 @@ int sys_set_ea(VFS_FUNC_ARGS_EA_SET)
 	LOG(log_debug, logtype_afpd, "sys_set_ea(%s): file is already opened", uname);
 	ret = sys_fsetxattr(fd, attruname, eabuf, attrsize, attr_flag);
     } else {
-	if ((oflag & O_NOFOLLOW) ) {
+	if (oflag & O_NOFOLLOW) {
    	    ret = sys_lsetxattr(uname, attruname, eabuf, attrsize,attr_flag);
 	}
 	else {
@@ -416,6 +424,8 @@ int sys_set_ea(VFS_FUNC_ARGS_EA_SET)
     }
     /* PBaranski fix */
 
+    free(eabuf);
+
     if (ret == -1) {
         switch(errno) {
         case OPEN_NOFOLLOW_ERRNO:
@@ -479,12 +489,16 @@ int sys_remove_ea(VFS_FUNC_ARGS_EA_REMOVE)
         return AFPERR_ACCESS;
 #endif
 
+    /* Protect special attributes set by Netatalk server */
+    if (!strncmp(attruname, AD_EA_META, AD_EA_META_LEN))
+	return AFPERR_ACCESS;
+	
     /* PBaranski fix */
     if ( fd != -1) {
 	LOG(log_debug, logtype_afpd, "sys_remove_ea(%s): file is already opened", uname);
 	ret = sys_fremovexattr(fd, uname, attruname);
     } else {
-	if ((oflag & O_NOFOLLOW) ) {
+	if (oflag & O_NOFOLLOW) {
     	    ret = sys_lremovexattr(uname, attruname);
 	}
 	else {
@@ -589,9 +603,6 @@ int sys_ea_copyfile(VFS_FUNC_ARGS_COPYFILE)
 		if (!*name)
 			continue;
 
-        if (STRCMP(name, ==, AD_EA_META))
-            continue;
-
 #if defined(SOLARIS) && defined(HAVE_SYS_ATTR_H)
         /* Skip special attributes set by NFS server */
         if (!strcmp(name, VIEW_READONLY) || !strcmp(name, VIEW_READWRITE))
